   /*This file is part of KnowMore.org Firefox Extension.

    KnowMore.org Firefox Extension is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License 
    as published by the Free Software Foundation, either version 3 of the 
    License, or (at your option) any later version.

    KnowMore.org Firefox Extension is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KnowMore.org Firefox Extension.  If not, see <http://www.gnu.org/licenses/>.*/
// namespace
this.KnowMore = this.KnowMore || {}

/**
 * KnowMore sidebar controller
 */
KnowMore.Sidebar = {};


/**
 * Getter for pointer to our tree
 */
KnowMore.Sidebar.__defineGetter__("tree",
    function(){ return document.getElementById("knowmore-sidebar-tree"); });

/**
 * Getter/Setter for tree mode (saved in preferences)
 */
KnowMore.Sidebar.__defineGetter__("mode",
    function() {
      return KnowMore.Utils.getPref("extensions.kinoradar.sidebar_treeview_mode",
                                    "alphabeta");
    });
KnowMore.Sidebar.__defineSetter__("mode",
    function(val) {
      KnowMore.Utils.setPref("extensions.kinoradar.sidebar_treeview_mode",
                             "char", val);
    });

/**
 * Fired when KnowMore sidebar is loaded
 */
KnowMore.Sidebar.onLoad = function()
{
  try {
    KnowMore.log("KnowMore: initializing KnowMore sidebar controller");
    
    KnowMore.DbService._construct();
    
    var profiles = KnowMore.DbService.getAllProfiles();
    KnowMore.log("Profiles count: " + profiles.length + ", Selected view mode: " + this.mode);
    this.mode = "normal";
    if(this.mode == "normal") {
      // Normal view mode
      this.tree.view = new KM_TreeviewPrototype(profiles);
    } else if(this.mode == "alphabeta") {
      // Alphabetical view mode
      this.tree.view = new KM_TreeviewPrototype(profiles);
    }
    else if(this.mode == "recent") {
      // Recent view mode
    this.tree.view = new KM_TreeviewPrototype(profiles);      
    }
    else {
      KnowMore.log("KnowMore.Sidebar: Unknow tree view mode: " + 
      KnowMore.Sidebar.mode);
    }
  } catch(e) {
    KnowMore.log("KnowMore.Sidebar.onLoad: " + e);
  }
};


/**
 * Set sidebar tree view mode [normal|alphabeta|recent]
 *
 * @param aMode {string}
 */
KnowMore.Sidebar.setTreeViewMode = function(aMode)
{
  if(aMode != "normal" || aMode != "alphabeta" || aMode != "recent") return;
  
  this.mode = aMode;
};


/**
 * Fired when user double-clicked on any tree item (company)
 */
KnowMore.Sidebar.onTreeDblclick = function()
{
  try {
    var uri  = this.tree.view.getCellValue(this.tree.currentIndex,
                                           this.tree.columns.getPrimaryColumn());
    
    KnowMore.log("KnowMore.Sidebar.onTreeDblclick: " + uri);
    openUILinkIn(uri, "tab", null, null, null);
  } catch(e) {
    KnowMore.log("KnowMore.Sidebar.onTreeDblclick: " + e);
  }
};


/**
 * Sidebar tree view prototype
 * 
 * @param aItems {array}
 */
var KM_TreeviewPrototype = function(aItems) {
  this.treeitems = aItems;
};
KM_TreeviewPrototype.prototype =
{
  treebox   : null,
  selection : null,
  
  get rowCount() { 
    return (KnowMore.Sidebar.mode == "alphabeta") 
        ? this.treeitems._correct_count + (122-97) 
        : this.treeitems.length; 
  },
  
  get atomSrv() {
    return Components.classes["@mozilla.org/atom-service;1"].
        getService(Components.interfaces.nsIAtomService);
  },
  
  
  getCellText : function(aRow, aCol) 
  {
    if(KnowMore.Sidebar.mode == "normal") {
      if(aCol.id == "knowmore-sidebar-tree-col0")
        return aRow + 1;
      
      if(aCol.id == "knowmore-sidebar-tree-col1")
        return this.treeitems[aRow].cmp_name;
      
      return "";
    }
    else if(KnowMore.Sidebar.mode == "alphabeta") {
      // ...
    }
    else if(KnowMore.Sidebar.mode == "recent") {
      // ...
    }
    else {
      KnowMore.log("KnowMore.Sidebar: Unknow tree view mode: " + 
      KnowMore.Sidebar.mode);
    }
  },
  
  getCellValue : function(aRow, aCol) {
    return this.treeitems[aRow].km_url;
  },
  
  hasNextSibling : function(aRow, aAfterIndex) {
    var thisLevel = this.getLevel(aRow);
    
    for(var i=(aRow+1); i<this.rowCount; i++) 
      if(this.getLevel(i) == thisLevel) return true;
    
    return false;
  },
  
  getCellProperties : function(aRow, aCol, aProp) { return; },
  getColumnProperties : function(aCol, aProp) { return; },
  getRowProperties : function(aRow, aProp) { return; },
  isContainer : function(aRow) { return false; },
  isContainerOpen : function(aRow) { return false; },
  isContainerEmpty : function(aRow) { return; },
  canDrop : function(aRow, aOrientation) { return true; },
  drop : function(aRow, aOrientation) {},
  setTree : function(aOut) { treebox = aOut; },
  getParentIndex : function(aRow) { return 0; },
  getLevel : function(aRow) { return 0; },
  getImageSrc : function(aRow, aCol) { return ""; },
  toggleOpenState : function(aIndex) {},
  isSorted : function() { return false; },
  cycleHeader : function(aCol) { return; },
  selectionChanged : function() {},
  cycleCell : function(aRow, aCol) {},
  isEditable : function(aRow, aCol) { return false; },
  setCellText : function(aRow, aCol, aValue) {},
  setCellValue : function(aRow, aCol, aValue) {},
  performAction : function(aAction) {},
  performActionOnRow : function(aAction, aRow) {},
  preformActionOnCell : function(aAction, aRow, aCol) {},
  isSeparator : function(aIndex) { return false; }
};
